package MakeWorld::Run;

use strict;
use warnings;

use File::Basename;

use MakeWorld::Global;
use MakeWorld::Log;

use Capture::Tiny qw[];

use base qw[ Exporter ];

our @EXPORT = qw[
		    run
		    capture
	       ];

#----------------------------------------------------------------#

# switches for programs which can handle "dry run" flags
our  %dryrun = (
                make       => '-n',
                gmake      => '-n',
                configure  => '-n',
               );

# wrapper for system() command which knows about dryruns
sub run
{
    my ( $exe, @args ) = @_;

    # @args may have an arrayref hidden in there, so take care of it
    @args = map { 'ARRAY' eq ref $_ ? @$_ : $_ } grep { defined $_ } @args;

    my $lexe = basename( $exe );

    unshift @args, $dryrun{$lexe}
      if $Opt->dryrun > 1 && exists $dryrun{$lexe};

    my $cmd = join(' ', $exe, @args );

    msg( $Opt->dryrun ? MakeWorld::Log::Info : MakeWorld::Log::Debug, 'executing: ' . $cmd );

    # this level of dryrun means just print what we'd do
    return 1 if 1 == $Opt->dryrun;

    # if a command can do a dryrun, have it do so, otherwise
    # return
    return 1 if $Opt->dryrun && ! exists $dryrun{$lexe};

    return system( $exe, @args ) == 0;
}


# wrapper for system() command which knows about dryruns
# FIXME: this is pretty much a duplicate of run()
sub capture
{
    my ( $exe, @args ) = @_;

    # @args may have an arrayref hidden in there, so take care of it
    @args = map { 'ARRAY' eq ref $_ ? @$_ : $_ } grep { defined $_ } @args;

    my $lexe = basename( $exe );

    unshift @args, $dryrun{$lexe}
      if $Opt->dryrun > 1 && exists $dryrun{$lexe};

    my $cmd = join(' ', $exe, @args );

    msg( $Opt->dryrun ? MakeWorld::Log::Info : MakeWorld::Log::Debug, 'executing: ' . $cmd );

    # this level of dryrun means just print what we'd do
    return 1 if 1 == $Opt->dryrun;

    # if a command can do a dryrun, have it do so, otherwise
    # return
    return 1 if $Opt->dryrun && ! exists $dryrun{$lexe};

    my $exit;
    my ( $stdout, $stderr ) = Capture::Tiny::capture { $exit = system( $exe, @args ) };

    return( $exit, $stdout, $stderr );
}


1;
