#!perl

use strict;
use warnings;


use Test::More;
use MakeWorld::Stages;

################################################################################
# Test default stages

my @default_stages = qw[ configure build test install ];


# [ filter ], [ expected ]

my @tpars = (

	     [
	      \@default_stages,
	      [],
	      \@default_stages,
	     ],

	     [
	      \@default_stages,
	      [ qw{ build test } ],
	      [ qw{ build test } ],
	     ],


	     [
	      \@default_stages,
	      [ qw{ -build -test } ],
	      [ qw{ configure install } ],
	     ],

	     [
	      [ qw{ configure build } ],
	      [ qw{ -build:pre } ],
	      [ qw{ configure build:main build:post } ],
	     ],
);


for my $tpar ( @tpars ) {

    test_filter( @$tpar );

}

sub expand {

    my $stage = shift;

    return map { "$stage:$_" } qw( pre main post );
}

sub test_filter {


    my ( $base_stages, $filter, $expected ) = @_;

    my $stages = MakeWorld::Stages->new( stages => $base_stages );

    # first make sure all stages work
    for my $stage ( @$base_stages )
    {
	ok( $stages->is_stage( $stage ), "is_stage: $stage" );
	ok( $stages->ok_to_run( $_ ), "ok_to_run: $_" )
	    for $stage, expand( $stage );

	ok( $stages->is_legal_hook( $_), "hook: $_" )
	    for "$stage:pre", "$stage:post";
    }

    is( $stages->last_stage, $base_stages->[-1], 'last stage' );


    # now test filters.

    # set up allowed stages
    $stages->allow_stages( $filter );

    # all hooks are still legal
    for my $stage ( @$base_stages )
    {
	ok( $stages->is_legal_hook( $_), "hook: $_" )
	    for "$stage:pre", "$stage:post";
    }


    # Set up map indicating whether a stage is possible
    # default to false
    my %possible = map { $_ => 0 } map { expand($_) } @$base_stages;

    # and set to true for the expected stages
    $possible{$_} = 1 foreach map { $_ =~ /:/ ? $_ : expand( $_ ) } @$expected;


    # and now test
    ok(  $stages->ok_to_run( $_ ) == $possible{$_}, "ok_to_run: $_ ($possible{$_})" )
      foreach sort keys %possible;

}


done_testing;
